require 'aws/ses'

module MailHandler
  # Containing everything related to SES comunication
  module SES
    # Setup and cofig the SES
    class << self
      attr_reader :mailer

      def setup(config)
        @mailer = Mailer.new config
      end
    end

    # Yields an interface to SES
    class Mailer
      def initialize(config)
        @ses = ::AWS::SES::Base.new(
          access_key_id:     config[:access_key_id],
          secret_access_key: config[:secret_access_key],
          server:            config[:ses_server]
        )
        check_connection
      end

      def send_mail(email)
        response = @ses.send_email email.to_h
        log email, response
        response
      rescue StandardError => e
        puts e.to_s
      end

      private

      def log(email, response)
        if response
          if response.error?
            log_failure email, response
          else
            log_success email
          end
        end
      end

      def log_failure(email, response)
        Logger.error(
          subject:    email,
          error_code: response.code,
          error_type: 'SES ERROR',
          trace:      response.error.to_s
        )
      end

      def log_success(email)
        Logger.success(
          subject: email,
          message: 'successfully sent'
        )
      end

      def check_connection
        @ses.quota
      end
    end
  end
end
