require 'aws-sdk'

module MailHandler
  # Contains everything rellated to SQS communication
  module SQS
    # Setup and cofig the sqs queue
    class << self
      attr_reader :client

      def setup(config)
        @client = Client.new config
      end
    end

    # Yields an interface to a specific sqs queue
    class Client
      attr_accessor :queues

      def initialize(config)
        all_queues = ::AWS::SQS.new(
          access_key_id:     config[:access_key_id],
          secret_access_key: config[:secret_access_key],
          region:            config[:aws_region]
        ).queues

        @queues = config[:sqs_queues].map do |queue_name|
          all_queues.named queue_name
        end

        @batch_size = config[:batch_size] || 1
      end

      def fetch_messages
        [
          @queues.map do |queue|
            queue.receive_messages(limit: @batch_size)
          end
        ].flatten.compact
      end
    end
  end
end
