require 'workers/threaded_worker'
require 'models/email'

module MailHandler
  # Base handler that does it all
  class Handler
    def self.begin_work_loop
      loop do
        begin
          new.do_work
        rescue StandardError => e
          Logger.error(
            subject: 'Main loop',
            error_type: e.class.to_s,
            trace: e.to_s
          )
          raise e
        end
      end
    end

    def do_work
      if messages.any?
        messages.each do |message|
          if response = send(email_from(message))
            message.delete unless response.error?
          end
        end
      else
        sleep 1
      end
    end

    private

    def messages
      @messages ||= SQS.client.fetch_messages
    end

    def email_from(message)
      MailHandler::Models::Email.new(
        JSON.parse(message.body, symbolize_names: true)
      )
    end

    def send(email)
      SES.mailer.send_mail email
    end

    def wait_for(worker)
      sleep(0.01) until worker.done?
    end
  end
end
